@@ -6,9 +6,28 @@
 // modified, or distributed except according to those terms. Please review the Licences for the
 // specific language governing permissions and limitations relating to use of the SAFE Network
 // Software.
+#![cfg_attr(docsrs, feature(doc_auto_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 
 #[cfg(not(windows))]
 mod posix;
+#[cfg(all(
+    not(windows),
+    not(all(
+        target_vendor = "apple",
+        any(
+            target_os = "macos",
+            target_os = "ios",
+            target_os = "tvos",
+            target_os = "watchos",
+            target_os = "visionos"
+        )
+    )),
+    not(target_os = "freebsd"),
+    not(target_os = "netbsd"),
+    not(target_os = "openbsd")
+))]
+mod posix_not_apple;
 mod sockaddr;
 #[cfg(windows)]
 mod windows;
@@ -25,21 +44,30 @@ pub struct Interface {
     pub addr: IfAddr,
     /// The index of the interface.
     pub index: Option<u32>,
+    /// (Windows only) A permanent and unique identifier for the interface. It
+    /// cannot be modified by the user. It is typically a GUID string of the
+    /// form: "{XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX}", but this is not
+    /// guaranteed by the Windows API.
+    #[cfg(windows)]
+    pub adapter_name: String,
 }
 
 impl Interface {
     /// Check whether this is a loopback interface.
-    pub fn is_loopback(&self) -> bool {
+    #[must_use]
+    pub const fn is_loopback(&self) -> bool {
         self.addr.is_loopback()
     }
 
     /// Check whether this is a link local interface.
-    pub fn is_link_local(&self) -> bool {
+    #[must_use]
+    pub const fn is_link_local(&self) -> bool {
         self.addr.is_link_local()
     }
 
     /// Get the IP address of this interface.
-    pub fn ip(&self) -> IpAddr {
+    #[must_use]
+    pub const fn ip(&self) -> IpAddr {
         self.addr.ip()
     }
 }
@@ -55,7 +83,8 @@ pub enum IfAddr {
 
 impl IfAddr {
     /// Check whether this is a loopback address.
-    pub fn is_loopback(&self) -> bool {
+    #[must_use]
+    pub const fn is_loopback(&self) -> bool {
         match *self {
             IfAddr::V4(ref ifv4_addr) => ifv4_addr.is_loopback(),
             IfAddr::V6(ref ifv6_addr) => ifv6_addr.is_loopback(),
@@ -63,7 +92,8 @@ impl IfAddr {
     }
 
     /// Check whether this is a link local interface.
-    pub fn is_link_local(&self) -> bool {
+    #[must_use]
+    pub const fn is_link_local(&self) -> bool {
         match *self {
             IfAddr::V4(ref ifv4_addr) => ifv4_addr.is_link_local(),
             IfAddr::V6(ref ifv6_addr) => ifv6_addr.is_link_local(),
@@ -71,7 +101,8 @@ impl IfAddr {
     }
 
     /// Get the IP address of this interface address.
-    pub fn ip(&self) -> IpAddr {
+    #[must_use]
+    pub const fn ip(&self) -> IpAddr {
         match *self {
             IfAddr::V4(ref ifv4_addr) => IpAddr::V4(ifv4_addr.ip),
             IfAddr::V6(ref ifv6_addr) => IpAddr::V6(ifv6_addr.ip),
@@ -86,18 +117,22 @@ pub struct Ifv4Addr {
     pub ip: Ipv4Addr,
     /// The netmask of the interface.
     pub netmask: Ipv4Addr,
+    /// The CIDR prefix of the interface.
+    pub prefixlen: u8,
     /// The broadcast address of the interface.
     pub broadcast: Option<Ipv4Addr>,
 }
 
 impl Ifv4Addr {
     /// Check whether this is a loopback address.
-    pub fn is_loopback(&self) -> bool {
-        self.ip.octets()[0] == 127
+    #[must_use]
+    pub const fn is_loopback(&self) -> bool {
+        self.ip.is_loopback()
     }
 
     /// Check whether this is a link local address.
-    pub fn is_link_local(&self) -> bool {
+    #[must_use]
+    pub const fn is_link_local(&self) -> bool {
         self.ip.is_link_local()
     }
 }
@@ -109,18 +144,22 @@ pub struct Ifv6Addr {
     pub ip: Ipv6Addr,
     /// The netmask of the interface.
     pub netmask: Ipv6Addr,
+    /// The CIDR prefix of the interface.
+    pub prefixlen: u8,
     /// The broadcast address of the interface.
     pub broadcast: Option<Ipv6Addr>,
 }
 
 impl Ifv6Addr {
     /// Check whether this is a loopback address.
-    pub fn is_loopback(&self) -> bool {
-        self.ip.segments() == [0, 0, 0, 0, 0, 0, 0, 1]
+    #[must_use]
+    pub const fn is_loopback(&self) -> bool {
+        self.ip.is_loopback()
     }
 
     /// Check whether this is a link local address.
-    pub fn is_link_local(&self) -> bool {
+    #[must_use]
+    pub const fn is_link_local(&self) -> bool {
         let bytes = self.ip.octets();
 
         bytes[0] == 0xfe && bytes[1] == 0x80
@@ -160,10 +199,15 @@ mod getifaddrs_posix {
                     } else {
                         None
                     };
-
+                    let prefixlen = if cfg!(target_endian = "little") {
+                        u32::from_le_bytes(netmask.octets()).count_ones() as u8
+                    } else {
+                        u32::from_be_bytes(netmask.octets()).count_ones() as u8
+                    };
                     IfAddr::V4(Ifv4Addr {
                         ip: ipv4_addr,
                         netmask,
+                        prefixlen,
                         broadcast,
                     })
                 }
@@ -180,10 +224,15 @@ mod getifaddrs_posix {
                     } else {
                         None
                     };
-
+                    let prefixlen = if cfg!(target_endian = "little") {
+                        u128::from_le_bytes(netmask.octets()).count_ones() as u8
+                    } else {
+                        u128::from_be_bytes(netmask.octets()).count_ones() as u8
+                    };
                     IfAddr::V6(Ifv6Addr {
                         ip: ipv6_addr,
                         netmask,
+                        prefixlen,
                         broadcast,
                     })
                 }
@@ -241,6 +290,7 @@ mod getifaddrs_windows {
                     Some(IpAddr::V4(ipv4_addr)) => {
                         let mut item_netmask = Ipv4Addr::new(0, 0, 0, 0);
                         let mut item_broadcast = None;
+                        let item_prefix = addr.OnLinkPrefixLength;
 
                         // Search prefixes for a prefix matching addr
                         'prefixloopv4: for prefix in ifaddr.prefixes() {
@@ -288,11 +338,13 @@ mod getifaddrs_windows {
                         IfAddr::V4(Ifv4Addr {
                             ip: ipv4_addr,
                             netmask: item_netmask,
+                            prefixlen: item_prefix,
                             broadcast: item_broadcast,
                         })
                     }
                     Some(IpAddr::V6(ipv6_addr)) => {
                         let mut item_netmask = Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 0);
+                        let item_prefix = addr.OnLinkPrefixLength;
                         // Search prefixes for a prefix matching addr
                         'prefixloopv6: for prefix in ifaddr.prefixes() {
                             let ipprefix = sockaddr::to_ipaddr(prefix.Address.lpSockaddr);
@@ -332,6 +384,7 @@ mod getifaddrs_windows {
                         IfAddr::V6(Ifv6Addr {
                             ip: ipv6_addr,
                             netmask: item_netmask,
+                            prefixlen: item_prefix,
                             broadcast: None,
                         })
                     }
@@ -345,6 +398,7 @@ mod getifaddrs_windows {
                     name: ifaddr.name(),
                     addr,
                     index,
+                    adapter_name: ifaddr.adapter_name(),
                 });
             }
         }
@@ -353,12 +407,138 @@ mod getifaddrs_windows {
     }
 }
 
+/// Get a list of all the network interfaces on this machine along with their IP info.
 #[cfg(windows)]
-/// Get address
 pub fn get_if_addrs() -> io::Result<Vec<Interface>> {
     getifaddrs_windows::get_if_addrs()
 }
 
+#[cfg(not(any(
+    all(
+        target_vendor = "apple",
+        any(
+            target_os = "macos",
+            target_os = "ios",
+            target_os = "tvos",
+            target_os = "watchos",
+            target_os = "visionos"
+        )
+    ),
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+)))]
+#[cfg_attr(
+    docsrs,
+    doc(cfg(any(
+        not(target_vendor = "apple"),
+        not(target_os = "freebsd"),
+        not(target_os = "netbsd"),
+        not(target_os = "openbsd")
+    )))
+)]
+mod if_change_notifier {
+    use super::Interface;
+    use std::collections::HashSet;
+    use std::io;
+    use std::time::{Duration, Instant};
+
+    #[derive(Debug, PartialEq, Eq, Hash, Clone)]
+    pub enum IfChangeType {
+        Added(Interface),
+        Removed(Interface),
+    }
+
+    #[cfg(windows)]
+    type InternalIfChangeNotifier = crate::windows::WindowsIfChangeNotifier;
+    #[cfg(not(windows))]
+    type InternalIfChangeNotifier = crate::posix_not_apple::PosixIfChangeNotifier;
+
+    /// (Not available on iOS/macOS) A utility to monitor for interface changes
+    /// and report them, so you can handle events such as WiFi
+    /// disconnection/flight mode/route changes
+    pub struct IfChangeNotifier {
+        inner: InternalIfChangeNotifier,
+        last_ifs: HashSet<Interface>,
+    }
+
+    impl IfChangeNotifier {
+        /// Create a new interface change notifier. Returns an OS specific error
+        /// if the network notifier could not be set up.
+        pub fn new() -> io::Result<Self> {
+            Ok(Self {
+                inner: InternalIfChangeNotifier::new()?,
+                last_ifs: HashSet::from_iter(super::get_if_addrs()?),
+            })
+        }
+
+        /// (Not available on iOS/macOS) Block until the OS reports that the
+        /// network interface list has changed, or until an optional timeout.
+        ///
+        /// For example, if an ethernet connector is plugged/unplugged, or a
+        /// WiFi network is connected to.
+        ///
+        /// The changed interfaces are returned. If an interface has both IPv4
+        /// and IPv6 addresses, you can expect both of them to be returned from
+        /// a single call to `wait`.
+        ///
+        /// Returns an [`io::ErrorKind::WouldBlock`] error on timeout, or
+        /// another error if the network notifier could not be read from.
+        pub fn wait(&mut self, timeout: Option<Duration>) -> io::Result<Vec<IfChangeType>> {
+            let start = Instant::now();
+            loop {
+                self.inner
+                    .wait(timeout.map(|t| t.saturating_sub(start.elapsed())))?;
+
+                // something has changed - now we find out what (or whether it was spurious)
+                let new_ifs = HashSet::from_iter(super::get_if_addrs()?);
+                let mut changes: Vec<IfChangeType> = new_ifs
+                    .difference(&self.last_ifs)
+                    .cloned()
+                    .map(IfChangeType::Added)
+                    .collect();
+                changes.extend(
+                    self.last_ifs
+                        .difference(&new_ifs)
+                        .cloned()
+                        .map(IfChangeType::Removed),
+                );
+                self.last_ifs = new_ifs;
+
+                if !changes.is_empty() {
+                    return Ok(changes);
+                }
+            }
+        }
+    }
+}
+
+#[cfg(not(any(
+    all(
+        target_vendor = "apple",
+        any(
+            target_os = "macos",
+            target_os = "ios",
+            target_os = "tvos",
+            target_os = "watchos",
+            target_os = "visionos"
+        )
+    ),
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+)))]
+#[cfg_attr(
+    docsrs,
+    doc(cfg(any(
+        not(target_vendor = "apple"),
+        not(target_os = "freebsd"),
+        not(target_os = "netbsd"),
+        not(target_os = "openbsd")
+    )))
+)]
+pub use if_change_notifier::{IfChangeNotifier, IfChangeType};
+
 #[cfg(test)]
 mod tests {
     use super::{get_if_addrs, Interface};
@@ -370,15 +550,15 @@ mod tests {
     use std::time::Duration;
 
     fn list_system_interfaces(cmd: &str, arg: &str) -> String {
-        let start_cmd = if arg == "" {
+        let start_cmd = if arg.is_empty() {
             Command::new(cmd).stdout(Stdio::piped()).spawn()
         } else {
             Command::new(cmd).arg(arg).stdout(Stdio::piped()).spawn()
         };
         let mut process = match start_cmd {
             Err(why) => {
-                println!("couldn't start cmd {} : {}", cmd, why.to_string());
-                return "".to_string();
+                println!("couldn't start cmd {} : {}", cmd, why);
+                return String::new();
             }
             Ok(process) => process,
         };
@@ -413,7 +593,7 @@ mod tests {
             .collect()
     }
 
-    #[cfg(any(target_os = "linux", target_os = "android", target_os = "nacl"))]
+    #[cfg(any(target_os = "linux", target_os = "android"))]
     fn list_system_addrs() -> Vec<IpAddr> {
         list_system_interfaces("ip", "addr")
             .lines()
@@ -429,7 +609,21 @@ mod tests {
             .collect()
     }
 
-    #[cfg(any(target_os = "freebsd", target_os = "macos", target_os = "ios"))]
+    #[cfg(any(
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd",
+        all(
+            target_vendor = "apple",
+            any(
+                target_os = "macos",
+                target_os = "ios",
+                target_os = "tvos",
+                target_os = "watchos",
+                target_os = "visionos"
+            )
+        )
+    ))]
     fn list_system_addrs() -> Vec<IpAddr> {
         list_system_interfaces("ifconfig", "")
             .lines()
@@ -484,4 +678,37 @@ mod tests {
             assert!(listed);
         }
     }
+
+    #[cfg(not(any(
+        all(
+            target_vendor = "apple",
+            any(
+                target_os = "macos",
+                target_os = "ios",
+                target_os = "tvos",
+                target_os = "watchos",
+                target_os = "visionos"
+            )
+        ),
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    )))]
+    #[test]
+    fn test_if_notifier() {
+        // Check that the interface notifier can start up and time out. No easy
+        // way to programmatically add/remove interfaces, so set a timeout of 0.
+        // Will cover a potential case of inadequate setup leading to an
+        // immediate change notification.
+        //
+        // There is a small race condition from creation -> check that an
+        // interface change *actually* occurs, so this test may spuriously fail
+        // extremely rarely.
+
+        let notifier = crate::IfChangeNotifier::new();
+        assert!(notifier.is_ok());
+        let mut notifier = notifier.unwrap();
+
+        assert!(notifier.wait(Some(Duration::ZERO)).is_err());
+    }
 }
